@@ -1,135 +1,29 @@
 """Contains all the commands that the CLI can execute."""
-import importlib
-import importlib.machinery
-import importlib.util
-import inspect
-import logging
 import os
 import os.path
 import pathlib
 import sys
 import typing as t
-from types import ModuleType
 
 import click
 
-from py_mono_tools.backends import Docker, System
-from py_mono_tools.config import consts, logger
-from py_mono_tools.goals import deployers as deployers_mod, linters as linters_mod, testers as testers_mod
-from py_mono_tools.goals.interface import Deployer, Language, Linter, Tester
+from py_mono_tools.cli_interface import GoalOutput
+from py_mono_tools.config import cfg, logger
+from py_mono_tools.goals.interface import Language
+from py_mono_tools.utils import (
+    filter_linters,
+    find_goals,
+    init_backend,
+    init_logger,
+    load_conf,
+    machine_goal_to_human_output,
+    set_absolute_path,
+    set_path_from_conf_name,
+    set_relative_path,
+)
 
 
-logger.info("Starting main")
-
-
-def _find_goals():
-    goals = [
-        (linters_mod, Linter, "linters", consts.ALL_LINTERS, consts.ALL_LINTER_NAMES),
-        (deployers_mod, Deployer, "deployers", consts.ALL_DEPLOYERS, consts.ALL_DEPLOYER_NAMES),
-        (testers_mod, Tester, "testers", consts.ALL_TESTERS, consts.ALL_TESTER_NAMES),
-    ]
-
-    for goal in goals:
-        goal_mod, goal_abc, goal_name, consts_goal_instances, consts_goal_names = goal
-        logger.debug("Finding %s", goal_name)
-        goal_classes = inspect.getmembers(goal_mod, inspect.isclass)
-        goal_instances = [
-            goal_class[1]()
-            for goal_class in goal_classes
-            if issubclass(goal_class[1], goal_abc) and goal_class[1] != goal_abc
-        ]
-        consts_goal_instances.extend(goal_instances)
-
-        for goal_instance in goal_instances:
-            consts_goal_names.append(goal_instance.name)
-
-    consts.ALL_BACKENDS = [Docker, System]
-    consts.ALL_BACKEND_NAMES = [Docker.name, System.name]
-
-
-_find_goals()
-
-
-def _load_conf(conf_path: str) -> ModuleType:
-    conf_location = f"{conf_path}/CONF"
-    logger.debug("CONF location: %s", conf_location)
-
-    loader = importlib.machinery.SourceFileLoader("CONF", conf_location)
-    spec = importlib.util.spec_from_loader(loader.name, loader)
-    mod = importlib.util.module_from_spec(spec)  # type: ignore
-    loader.exec_module(mod)
-
-    return mod
-
-
-def _init_logger(verbose: bool):
-    if verbose:
-        logger.setLevel(logging.DEBUG)
-        for handler in logger.handlers:
-            handler.setLevel(logging.DEBUG)
-
-
-def _set_absolute_path(absolute_path: str):
-    logger.info("Overwriting execution root path: %s", absolute_path)
-
-    consts.EXECUTED_FROM = pathlib.Path(absolute_path).resolve()
-
-    os.chdir(consts.EXECUTED_FROM.resolve())
-
-
-def _set_relative_path(relative_path: str):
-    logger.info("Overwriting execution root path: %s", relative_path)
-
-    consts.EXECUTED_FROM = consts.EXECUTED_FROM.joinpath(pathlib.Path(relative_path).resolve())
-    os.chdir(consts.EXECUTED_FROM)
-
-
-def _set_path_from_conf_name(name: str):
-    logger.info("Setting path from conf name: %s", name)
-
-    for rel_path, _, filenames in os.walk("."):
-        for filename in filenames:
-            if filename == "CONF":
-                path = pathlib.Path(rel_path).resolve()
-                mod = _load_conf(str(path))
-                logger.debug("Found CONF: %s in %s", mod.NAME, path)
-                if mod.NAME.strip().lower() == name.strip().lower():
-                    logger.debug("Using CONF: %s in %s", mod.NAME, path)
-                    _set_absolute_path(str(path))
-                    return
-
-
-def _init_backend(_build_system: str):
-    logger.debug("Initializing build system: %s", _build_system)
-
-    consts.BACKENDS = {
-        Docker.name: Docker,
-        System.name: System,
-    }
-
-    consts.CURRENT_BACKEND = consts.BACKENDS[_build_system]()
-
-
-def _filter_linters(specific_linters: t.List[str], language: t.Optional[Language]) -> t.List[Linter]:
-    conf_linters: t.List[Linter] = consts.CONF.LINT  # type: ignore
-    logger.debug("Linters: %s", conf_linters)
-    linters_to_run: t.List[Linter] = []
-    specific_linters = [s.strip().lower() for s in specific_linters]
-
-    if language is None and not specific_linters:
-        logger.debug("No language or specific linters specified. Running all linters.")
-        linters_to_run = conf_linters
-    else:
-        logger.debug("Language: %s, specific: %s", language, specific_linters)
-        for linter in conf_linters:
-            if linter.name.strip().lower() in specific_linters:
-                linters_to_run.append(linter)
-                continue
-            if linter.language == language:
-                linters_to_run.append(linter)
-
-    logger.debug("Linters to run: %s", linters_to_run)
-    return linters_to_run
+find_goals()
 
 
 @click.group()
@@ -146,40 +40,59 @@ This can be set via this flag, the BACKEND var in CONF, or defaulted to system.
 @click.option("--relative_path", "-rp", default=None, type=click.Path())
 @click.option("--name", "-n", default=None, type=str, help="Name as defined in CONF NAME=...")
 @click.option("--verbose", "-v", default=False, is_flag=True)
-def cli(backend, absolute_path, relative_path, name, verbose):
+@click.option("--silent", "-s", default=False, is_flag=True)
+@click.option("--machine_output", "-mo", default=False, is_flag=True)
+# pylint: disable-next=R0913
+def cli(backend, absolute_path, relative_path, name, verbose, silent, machine_output):  # noqa: C901
     """Py mono tool is a CLI tool that simplifies using python in a monorepo."""
-    logger.info("Starting cli")
-
     if "--help" in sys.argv or "-h" in sys.argv:
         return
 
-    _init_logger(verbose=verbose)
+    if machine_output is True:
+        silent = True
+        verbose = False
+        cfg.USE_MACHINE_OUTPUT = True
 
-    logger.debug("Executed from: %s", consts.EXECUTED_FROM)
-    logger.debug("Current backend: %s", consts.CURRENT_BACKEND)
-    logger.debug("Backends: %s", consts.BACKENDS)
+    init_logger(verbose=verbose, silent=silent)
+    logger.info("Starting py_mono_tools")
+
+    logger.debug("Executed from: %s", cfg.EXECUTED_FROM)
+    logger.debug("Current backend: %s", cfg.CURRENT_BACKEND)
+    logger.debug("Backends: %s", cfg.BACKENDS)
 
     if absolute_path is not None:
-        _set_absolute_path(absolute_path=absolute_path)
+        set_absolute_path(absolute_path=absolute_path)
     elif relative_path is not None:
-        _set_relative_path(relative_path=relative_path)
+        set_relative_path(relative_path=relative_path)
     elif name is not None:
-        _set_path_from_conf_name(name)
+        set_path_from_conf_name(name)
 
     try:
-        mod = _load_conf(consts.EXECUTED_FROM)
-        consts.CONF = mod
+        mod = load_conf(cfg.EXECUTED_FROM)
+        cfg.CONF = mod
         if backend is None:
             try:
-                backend = consts.CONF.BACKEND
+                backend = cfg.CONF.BACKEND
             except AttributeError:
                 backend = "system"
 
         logger.info("Using backed: %s", backend)
 
-        _init_backend(backend)
+        init_backend(backend)
     except FileNotFoundError:
-        logger.error("No CONF file found in %s", consts.EXECUTED_FROM)
+        logger.error("No CONF file found in %s", cfg.EXECUTED_FROM)
+
+
+@cli.result_callback()
+def output(*args, **kwargs):  # pylint: disable=W0613
+    """
+    Will run after all commands.
+
+    Takes the machine output, converts to JSON, prints it, and exits.
+    """
+    if cfg.USE_MACHINE_OUTPUT is True:
+        click.echo(cfg.MACHINE_OUTPUT.json(indent=2))
+    sys.exit(cfg.MACHINE_OUTPUT.returncode)
 
 
 @cli.command()
@@ -193,9 +106,9 @@ def cli(backend, absolute_path, relative_path, name, verbose):
     Specify one or more linters to run. NOTE: The linter MUST be listed in the respected CONF file.
 
     All Linters:
-    {consts.ALL_LINTER_NAMES}
+    {cfg.ALL_LINTER_NAMES}
     """,
-    shell_complete=consts.ALL_LINTER_NAMES,
+    shell_complete=cfg.ALL_LINTER_NAMES,
 )
 @click.option("--fail_fast", "-ff", is_flag=True, default=False, help="Stop on first failure.")
 @click.option("--show_success", is_flag=True, default=False, help="Show successful outputs")
@@ -238,7 +151,7 @@ def lint(
 
     logger.info("Starting lint")
 
-    linters_to_run = _filter_linters(specific_linters=specific, language=language)
+    linters_to_run = filter_linters(specific_linters=specific, language=language)
 
     if ignore_linter_weight is False:
         linters_to_run.sort(key=lambda x: x.weight, reverse=True)
@@ -249,17 +162,26 @@ def lint(
             logs, return_code = linter.check()
         else:
             logs, return_code = linter.run()
+
         logger.info("Lint result: %s %s", linter.name, return_code)
 
-        if show_success is False and return_code == 0:
-            logger.debug("Skipping successful output")
-            logger.debug(logs)
-        else:
-            logger.info(logs)
+        goal = GoalOutput(name=linter.name, output=logs, returncode=return_code)
+        cfg.MACHINE_OUTPUT.goals[linter.name] = goal
+
+        if return_code != 0:
+            cfg.MACHINE_OUTPUT.returncode = 1
+
+        if cfg.USE_MACHINE_OUTPUT is False:
+            formatted_log = machine_goal_to_human_output(goal)
+            if show_success is False and return_code == 0:
+                logger.debug("Skipping successful output")
+                logger.debug(formatted_log)
+            else:
+                logger.info(formatted_log)
 
         if fail_fast is True and return_code != 0:
             logger.error("Linter %s failed with code %s", linter.name, return_code)
-            sys.exit(1)
+            sys.exit(return_code)
 
     logger.info("Linting complete")
 
@@ -267,7 +189,7 @@ def lint(
 @cli.command()
 def test():
     """Run all the tests specified in the CONF file."""
-    testers = consts.CONF.TEST
+    testers = cfg.CONF.TEST
     for tester in testers:
         logger.info("Testing: %s", tester.name)
         logs, return_code = tester.run()
@@ -279,7 +201,7 @@ def test():
 @click.option("--plan", is_flag=True, default=False)
 def deploy(plan: bool):
     """Run the specified build and deploy in the specific CONF file."""
-    deployers = consts.CONF.DEPLOY  # type: ignore
+    deployers = cfg.CONF.DEPLOY  # type: ignore
     for deployer in deployers:
         logger.info("Deploying: %s", deployer.name)
         if plan is True:
@@ -293,7 +215,7 @@ def deploy(plan: bool):
 @cli.command()
 def interactive():
     """Drop into an interactive session in your specified backend."""
-    consts.CURRENT_BACKEND.interactive()
+    cfg.CURRENT_BACKEND.interactive()
 
 
 @cli.command(name="list")
@@ -304,11 +226,11 @@ def list_():
         for filename in filenames:
             if filename == "CONF":
                 path = pathlib.Path(rel_path).resolve()
-                mod = _load_conf(str(path))
+                mod = load_conf(str(path))
                 conf_names.append(f"{mod.NAME} -- {rel_path}")
 
     print("Backends:")
-    for backend in consts.ALL_BACKEND_NAMES:
+    for backend in cfg.ALL_BACKEND_NAMES:
         print("    " + backend)
 
     print("CONF file names:")
@@ -316,15 +238,15 @@ def list_():
         print("    " + conf_name)
 
     print("Deployers:")
-    for deployer in consts.ALL_DEPLOYER_NAMES:
+    for deployer in cfg.ALL_DEPLOYER_NAMES:
         print("    " + deployer)
 
     print("Linters:")
-    for linter in consts.ALL_LINTER_NAMES:
+    for linter in cfg.ALL_LINTER_NAMES:
         print("    " + linter)
 
     print("Testers:")
-    for tester in consts.ALL_TESTER_NAMES:
+    for tester in cfg.ALL_TESTER_NAMES:
         print("    " + tester)
 
 
@@ -338,7 +260,7 @@ def list_():
 #     Docker is the default.
 #     """
 #     logger.info(modules)
-#     consts.CURRENT_BACKEND.build(force_rebuild=force_rebuild)
+#     cfg.CURRENT_BACKEND.build(force_rebuild=force_rebuild)
 #
 #
 #
